import uvicorn
from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
from starlette.responses import JSONResponse

from app.api.v1.api import api_router
from app.core.config import settings
from app.core.docs import api_description, external_docs, tags_metadata

app = FastAPI(
    title=settings.PROJECT_NAME,
    version=settings.VERSION,
    description=api_description,
    openapi_url="/openapi.json",
    docs_url="/docs",
    redoc_url="/redoc",
    openapi_tags=tags_metadata,
    openapi_extra={"externalDocs": external_docs.dict()},
)

# Add CORS middleware
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

app.include_router(api_router, prefix=settings.API_V1_STR)


@app.get("/", tags=["Root"])
async def root():
    """
    Root endpoint that returns basic API information
    """
    return JSONResponse(
        content={
            "title": settings.PROJECT_NAME,
            "docs": "/docs",
            "health": "/health",
        }
    )


@app.get("/health", tags=["Health"])
async def health_check():
    """
    Health check endpoint to verify the service is running
    """
    return JSONResponse(
        content={
            "status": "healthy",
            "version": settings.VERSION,
        }
    )


if __name__ == "__main__":
    uvicorn.run("main:app", host="0.0.0.0", port=8000, reload=True)
